from almost_p2p_chat.my_socket import *
import threading
import queue

UDP_MAX_SIZE = 65535


class Server:
    def __init__(self, host: str = '127.0.0.1', port: int = 3000):
        self.host = host
        self.port = port
        self.members = {}
        self.my_socket = MySocket()
        self.my_socket.my_bind(host, port)
        self.return_message = ""
        self.q = queue.Queue()
        # self.window = gui_window

    def run(self):
        self.return_message = f'Listening at {self.host}:{self.port} \n'
        print(self.return_message)
        self.q.put(self.return_message)
        while True:
            msg, addr = self.my_socket.my_recvfrom(1024)

            if addr not in self.members:
                self.members.setdefault(msg[1], addr)

            if not msg:
                continue

            msg_text = msg[0]
            if msg_text == '__join':
                self.return_message = f'Client: {msg[1]} joined chat \n'
                print(self.return_message)
                self.q.put(self.return_message)

                continue

            if msg_text == '__members':
                self.return_message = f'Client: {msg[1]} requsted members \n'
                self.q.put(self.return_message)

                print(self.return_message)

                active_members = [(m, v) for m, v in self.members.items() if m != msg[1]]
                self.my_socket.my_sendto(dict(active_members), addr)
                continue

            if msg_text == "__exit":
                self.return_message = f'Client: {msg[1]} leave chat \n'
                self.q.put(self.return_message)

                print(self.return_message)
                self.members.pop(msg[1])

    def start_run(self):
        self.run_thread = threading.Thread(target=self.run)
        self.run_thread.start()



import threading
import time
from my_socket import *
import message
import queue
import os
import pickle


class Client:
    UDP_MAX_SIZE = 65535

    def __init__(self, username: str, host: str = '127.0.0.1', port: int = 3000):
        self.host = host
        self.port = port
        self.username = username
        self.my_socket = MySocket()
        self.my_socket.my_bind(host, port)
        # self.own_port = self.my_socket.my_getsockname()[1]
        self.listen_thread = None
        self.sendto = None
        self.allowed_ports = [port]
        self.data_from_server = None
        self.window = None
        self.my_queue = queue.Queue()
        self.client_connections = {}

    def listen(self):
        while True:
            msg, addr = self.my_socket.my_recvfrom(self.UDP_MAX_SIZE)
            self.data_from_server = msg
            msg_port = addr[-1]
            allowed_ports = self.listen_thread.allowed_ports
            if msg_port not in allowed_ports:
                continue

            if not msg:
                continue

            else:
                self.my_queue.put(str(msg))
                self.client_connections[self.key_connection].append(msg)
                print('\r\r' + str(msg) + '\n' + f'you: ', end='')

    def start_listen(self):
        self.listen_thread = threading.Thread(target=self.listen, daemon=True)
        self.listen_thread.allowed_ports = self.allowed_ports
        self.listen_thread.start()

    def connect(self, host: str, port: int):
        self.start_listen()
        self.my_socket.my_sendto(('__join', self.username), (host, port))
        self.sendto = (host, port)

    def get_members(self):
        self.my_socket.my_sendto(('__members', self.username), self.sendto)
        time.sleep(0.01)
        return dict(self.data_from_server)

    def send_message(self, text):
        self.my_queue.put(f"you: {text}")
        msg = message.Message(self.username, text)
        self.client_connections[self.key_connection].append(msg)
        self.my_socket.my_sendto(msg, self.sendto)

    def connect_by_name(self, username: str):

        self.get_members()
        time.sleep(0.001)
        if dict(self.data_from_server).get(username):
            addr = dict(self.data_from_server).get(username)
            self.key_connection = f"{self.username}-{username}"
            self.client_connections.setdefault(self.key_connection)
            self.client_connections[self.key_connection] = list()

            if os.path.isfile(self.key_connection):
                with open(self.key_connection, 'rb') as file:
                    print("Ok "*50)
                    deserialized_message = pickle.load(file)
                    print(deserialized_message)
                    for i in deserialized_message:
                        self.client_connections[self.key_connection].append(i)
                        if i.sender == self.username:
                            self.my_queue.put(f"you: {i.text}")
                        else:
                            self.my_queue.put(str(i))

            with open(self.key_connection, 'wb') as file:
                pass

            host = addr[0]
            port = addr[1]
            peer_port = port
            self.allowed_ports.append(peer_port)
            self.sendto = (host, peer_port)
            print(f'Connect to client: {username}')
        else:
            print(f'Failed to connect')

    def find_username_port(self, port: int):
        b = self.get_members()
        a = dict(self.data_from_server)
        for key, val in a.items():
            if val[1] == port:
                return key

    def disconect_from_client(self):

        self.sendto = ("127.0.0.1", 3000)
        with open(self.key_connection, 'wb') as file:
            pickle.dump(self.client_connections.get(self.key_connection), file)



from almost_p2p_chat import client


class User():
    def __init__(self, username):
        self.username = username
        self._is_server = False
        self._is_client = False
        # self.host = host
        # self.port = port



    def became_client(self, host, port):
        self.client = client.Client(self.username, host, port)
        self._is_client = True
        print(f"{self.username} is now a client")

    def request_users(self):
        self.client.get_members()
        print(self.client.data_from_server)
        if self.client.data_from_server:
            return self.client.data_from_server
        else:
            return

    def connect_to_user_by_name(self, username: str):
        self.client.connect_by_name(username)

    def connect(self, host, port):
        self.client.connect(host, port)

    def disconnect(self):
        pass

    def disconnect_from_server(self):
        pass

    def is_server(self):
        return self._is_server

    def is_client(self):
        return self._is_client



import datetime
import pickle


class Message:

    def __init__(self, sender, text):
        self.sender = sender
        self.text = text
        self.date = datetime.datetime.now()

    def __str__(self):
        tmp = str(self.date).split(' ')
        date = tmp[0]
        time = tmp[1].split('.')[0]
        return f"{date} {self.sender} \t {self.text} \n{time} \n"



from setuptools import setup, find_packages

setup(
    name="almost_p2p_chat",
    version="0.0.2",
    description="library for p2p_chat",
    author="Eugene-Lasevich",
    author_email="lasevich009@gmail.com",
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.11",
        "Operating System :: OS Independent"
    ],
    packages=["almost_p2p_chat"],
    include_package_data=True
)


import pickle
from socket import socket
from socket import AF_INET
from socket import SOCK_STREAM
from socket import SOCK_DGRAM


class MySocket(socket):

    def __init__(self, sock=None):
        super().__init__()
        if sock is None:
            self.sock = socket(AF_INET, SOCK_DGRAM)
        else:
            self.sock = sock

    def my_connect(self, host, port):
        self.sock.connect((host, port))

    def my_bind(self, host, port):
        self.sock.bind((host, port))

    def my_listen(self):
        self.sock.listen()

    def my_accept(self):
        client_sock, client_addr = self.sock.accept()
        return MySocket(client_sock), client_addr

    def my_send(self, data):
        serialized_data = pickle.dumps(data)
        self.sock.sendall(serialized_data)

    def my_recv(self, bufsize):
        serialized_data = self.sock.recv(bufsize)
        if len(serialized_data) == 0:
            return
        data = pickle.loads(serialized_data)
        return data

    def my_recvfrom(self, bufsize):
        tmp, t = self.sock.recvfrom(bufsize)
        data = pickle.loads(tmp)
        return data, t

    def my_sendto(self, data, addr):
        serialized_data = pickle.dumps(data)
        self.sock.sendto(serialized_data, addr)

    def my_getsockname(self):
        return self.sock.getsockname()

    def my_close(self):
        self.sock.close()

    # def __del__(self):
    #     self.sock.close
    #     print("Socker was deleted")
